/****************************************************************************
** Meta object code from reading C++ file 'qgfxsourceproxy_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.13)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../git/src/effects/private/qgfxsourceproxy_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qgfxsourceproxy_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.13. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QGfxSourceProxy_t {
    QByteArrayData data[18];
    char stringdata0[235];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QGfxSourceProxy_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QGfxSourceProxy_t qt_meta_stringdata_QGfxSourceProxy = {
    {
QT_MOC_LITERAL(0, 0, 15), // "QGfxSourceProxy"
QT_MOC_LITERAL(1, 16, 12), // "inputChanged"
QT_MOC_LITERAL(2, 29, 0), // ""
QT_MOC_LITERAL(3, 30, 13), // "outputChanged"
QT_MOC_LITERAL(4, 44, 17), // "sourceRectChanged"
QT_MOC_LITERAL(5, 62, 13), // "activeChanged"
QT_MOC_LITERAL(6, 76, 20), // "interpolationChanged"
QT_MOC_LITERAL(7, 97, 8), // "repolish"
QT_MOC_LITERAL(8, 106, 5), // "input"
QT_MOC_LITERAL(9, 112, 11), // "QQuickItem*"
QT_MOC_LITERAL(10, 124, 6), // "output"
QT_MOC_LITERAL(11, 131, 10), // "sourceRect"
QT_MOC_LITERAL(12, 142, 6), // "active"
QT_MOC_LITERAL(13, 149, 13), // "interpolation"
QT_MOC_LITERAL(14, 163, 13), // "Interpolation"
QT_MOC_LITERAL(15, 177, 16), // "AnyInterpolation"
QT_MOC_LITERAL(16, 194, 20), // "NearestInterpolation"
QT_MOC_LITERAL(17, 215, 19) // "LinearInterpolation"

    },
    "QGfxSourceProxy\0inputChanged\0\0"
    "outputChanged\0sourceRectChanged\0"
    "activeChanged\0interpolationChanged\0"
    "repolish\0input\0QQuickItem*\0output\0"
    "sourceRect\0active\0interpolation\0"
    "Interpolation\0AnyInterpolation\0"
    "NearestInterpolation\0LinearInterpolation"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QGfxSourceProxy[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       5,   50, // properties
       1,   70, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   44,    2, 0x06 /* Public */,
       3,    0,   45,    2, 0x06 /* Public */,
       4,    0,   46,    2, 0x06 /* Public */,
       5,    0,   47,    2, 0x06 /* Public */,
       6,    0,   48,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       7,    0,   49,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,

 // properties: name, type, flags
       8, 0x80000000 | 9, 0x0049510f,
      10, 0x80000000 | 9, 0x00495009,
      11, QMetaType::QRectF, 0x00495103,
      12, QMetaType::Bool, 0x00495001,
      13, 0x80000000 | 14, 0x0049510b,

 // properties: notify_signal_id
       0,
       1,
       2,
       3,
       4,

 // enums: name, alias, flags, count, data
      14,   14, 0x0,    3,   75,

 // enum data: key, value
      15, uint(QGfxSourceProxy::AnyInterpolation),
      16, uint(QGfxSourceProxy::NearestInterpolation),
      17, uint(QGfxSourceProxy::LinearInterpolation),

       0        // eod
};

void QGfxSourceProxy::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QGfxSourceProxy *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->inputChanged(); break;
        case 1: _t->outputChanged(); break;
        case 2: _t->sourceRectChanged(); break;
        case 3: _t->activeChanged(); break;
        case 4: _t->interpolationChanged(); break;
        case 5: _t->repolish(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QGfxSourceProxy::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QGfxSourceProxy::inputChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QGfxSourceProxy::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QGfxSourceProxy::outputChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QGfxSourceProxy::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QGfxSourceProxy::sourceRectChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QGfxSourceProxy::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QGfxSourceProxy::activeChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QGfxSourceProxy::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QGfxSourceProxy::interpolationChanged)) {
                *result = 4;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickItem* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QGfxSourceProxy *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QQuickItem**>(_v) = _t->input(); break;
        case 1: *reinterpret_cast< QQuickItem**>(_v) = _t->output(); break;
        case 2: *reinterpret_cast< QRectF*>(_v) = _t->sourceRect(); break;
        case 3: *reinterpret_cast< bool*>(_v) = _t->isActive(); break;
        case 4: *reinterpret_cast< Interpolation*>(_v) = _t->interpolation(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QGfxSourceProxy *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setInput(*reinterpret_cast< QQuickItem**>(_v)); break;
        case 2: _t->setSourceRect(*reinterpret_cast< QRectF*>(_v)); break;
        case 4: _t->setInterpolation(*reinterpret_cast< Interpolation*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
        QGfxSourceProxy *_t = static_cast<QGfxSourceProxy *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->resetInput(); break;
        default: break;
        }
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QGfxSourceProxy::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickItem::staticMetaObject>(),
    qt_meta_stringdata_QGfxSourceProxy.data,
    qt_meta_data_QGfxSourceProxy,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QGfxSourceProxy::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QGfxSourceProxy::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QGfxSourceProxy.stringdata0))
        return static_cast<void*>(this);
    return QQuickItem::qt_metacast(_clname);
}

int QGfxSourceProxy::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 6;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 5;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QGfxSourceProxy::inputChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QGfxSourceProxy::outputChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QGfxSourceProxy::sourceRectChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QGfxSourceProxy::activeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QGfxSourceProxy::interpolationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
